//---------------------------------------------------------------------------

#include <Registry.hpp>
#pragma hdrstop

#include "winutil.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
#include <windows.h>
#include <stdio.h>

#define ACCESS_READ  1
#define ACCESS_WRITE 2

bool IsAdmin()
{
 HANDLE hToken;
 DWORD  dwStatus;
 DWORD  dwAccessMask;
 DWORD  dwAccessDesired;
 DWORD  dwACLSize;
 DWORD  dwStructureSize = sizeof(PRIVILEGE_SET);
 PACL   pACL            = NULL;
 PSID   psidAdmin       = NULL;
 BOOL   bReturn         = FALSE;
 PRIVILEGE_SET   ps;
 GENERIC_MAPPING GenericMapping;
 PSECURITY_DESCRIPTOR     psdAdmin           = NULL;
 SID_IDENTIFIER_AUTHORITY SystemSidAuthority = SECURITY_NT_AUTHORITY;
 class ELeaveNow {};
 try {
     if(ImpersonateSelf(SecurityImpersonation))
     {
       if (!OpenThreadToken(GetCurrentThread(), TOKEN_QUERY, FALSE, &hToken))
       {
        if (GetLastError() != ERROR_NO_TOKEN)
           throw ELeaveNow();

        if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
         throw ELeaveNow();

        if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
         throw ELeaveNow();
       }

       if (!AllocateAndInitializeSid(&SystemSidAuthority, 2,
        SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS,
        0, 0, 0, 0, 0, 0, &psidAdmin))
        throw ELeaveNow();

       psdAdmin = LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH);
       if (psdAdmin == NULL)
        throw ELeaveNow();

       if (!InitializeSecurityDescriptor(psdAdmin,
        SECURITY_DESCRIPTOR_REVISION))
        throw ELeaveNow();

       dwACLSize = sizeof(ACL) + sizeof(ACCESS_ALLOWED_ACE) +
           GetLengthSid(psidAdmin) - sizeof(DWORD);

       pACL = (PACL)LocalAlloc(LPTR, dwACLSize);
       if (pACL == NULL)
        throw ELeaveNow();

       if (!InitializeAcl(pACL, dwACLSize, ACL_REVISION2))
        throw ELeaveNow();

       dwAccessMask= ACCESS_READ | ACCESS_WRITE;

       if (!AddAccessAllowedAce(pACL, ACL_REVISION2, dwAccessMask, psidAdmin))
        throw ELeaveNow();

       if (!SetSecurityDescriptorDacl(psdAdmin, TRUE, pACL, FALSE))
        throw ELeaveNow();

       if(!SetSecurityDescriptorGroup(psdAdmin, psidAdmin, FALSE))
        throw ELeaveNow();
       if(!SetSecurityDescriptorOwner(psdAdmin, psidAdmin, FALSE))
        throw ELeaveNow();

       if (!IsValidSecurityDescriptor(psdAdmin))
        throw ELeaveNow();

       dwAccessDesired = ACCESS_READ;

       GenericMapping.GenericRead    = ACCESS_READ;
       GenericMapping.GenericWrite   = ACCESS_WRITE;
       GenericMapping.GenericExecute = 0;
       GenericMapping.GenericAll     = ACCESS_READ | ACCESS_WRITE;

       if (!AccessCheck(psdAdmin, hToken, dwAccessDesired,
        &GenericMapping, &ps, &dwStructureSize, &dwStatus, &bReturn))
        throw ELeaveNow();

       if(!RevertToSelf())
        bReturn = true;
     }
   } catch (...) {
   }


 if (pACL) LocalFree(pACL);
 if (psdAdmin) LocalFree(psdAdmin);
 if (psidAdmin) FreeSid(psidAdmin);

 return bReturn;
}


//---------------------------------------------------------------------------
// Registry Functions:
//---------------------------------------------------------------------------



int GetRegInt(AnsiString name)
{

  int i=0;
  TRegistry *Registry = new TRegistry(KEY_READ);
  try
  {
    if (Registry->KeyExists("Software")) {
       Registry->OpenKey("Software",false);
       if (Registry->KeyExists("The Hovgaard Klan")) {
         Registry->OpenKey("The Hovgaard Klan",false);
         if (Registry->KeyExists("HovText")) {
            Registry->OpenKey("HovText",false);
            if (Registry->ValueExists(name))
               i = Registry->ReadInteger(name);
         }
       }
    }
  }
  catch (...)
  {
  }
  delete Registry;
  return i;
}


void SetRegInt(AnsiString name, int i)
{
  TRegistry *Registry = new TRegistry(KEY_WRITE);
  try
  {
    Registry->OpenKey("Software",true);
    Registry->OpenKey("The Hovgaard Klan",true);
    Registry->OpenKey("HovText",true);
    Registry->WriteInteger(name, i);
  }
  catch (...)
  {
  }
  delete Registry;
}


void InstallStartup()
{
//HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Run
  TRegistry *Registry = new TRegistry(KEY_WRITE);
  try
  {
    Registry->OpenKey("Software",true);
    Registry->OpenKey("Microsoft",true);
    Registry->OpenKey("Windows",true);
    Registry->OpenKey("CurrentVersion",true);
    Registry->OpenKey("Run",true);
    Registry->WriteString("HovText", ParamStr(0));
  }
  catch (...)
  {
  }
  delete Registry;
}

bool HasStartup()
{
  bool StartUp=false;
  TRegistry *Registry = new TRegistry(KEY_READ);
  try
  {
    Registry->OpenKey("Software",false);
    Registry->OpenKey("Microsoft",false);
    Registry->OpenKey("Windows",false);
    Registry->OpenKey("CurrentVersion",false);
    Registry->OpenKey("Run",false);
    StartUp=Registry->ValueExists("HovText");
  }
  catch (...)
  {
  }
  delete Registry;
  return StartUp;
}

void RemoveStartup()
{
  TRegistry *Registry = new TRegistry(KEY_WRITE);
  try
  {
    Registry->OpenKey("Software",false);
    Registry->OpenKey("Microsoft",false);
    Registry->OpenKey("Windows",false);
    Registry->OpenKey("CurrentVersion",false);
    Registry->OpenKey("Run",false);
    Registry->DeleteValue("HovText");
  }
  catch (...)
  {
  }
  delete Registry;
}

